@@ -24,6 +24,7 @@ from bleak.backends.characteristic import (  # type: ignore[import]
     GattCharacteristicsFlags,
 )
 from bleak.backends.device import BLEDevice  # type: ignore[import]
+from bleak.backends.scanner import AdvertisementData  # type: ignore[import]
 from bleak.backends.service import BleakGATTService  # type: ignore[import]
 
 
@@ -71,6 +72,7 @@ class Adapter:  # pylint: disable=too-many-instance-attributes
         # Not known yet.
         self._hcitool_is_usable = None
         self._hcitool = None
+        self._hcidump = None
         self.ble_backend = None
 
         # Keep a cache of recently scanned devices, to avoid doing double
@@ -247,16 +249,14 @@ class Adapter:  # pylint: disable=too-many-instance-attributes
         while self._scanning_in_progress and (
             timeout is None or time.time() - start < timeout
         ):
-            for device in self.await_bleak(
+            for (device, advertisement_data) in self.await_bleak(
                 self._scan_for_interval(self._SCAN_INTERVAL)
             ):
-                if not device or (
-                    device.rssi is not None and device.rssi < minimum_rssi
-                ):
+                if advertisement_data.rssi < minimum_rssi:
                     continue
                 self._cache_device(device)
                 scan_entry = ScanEntry._from_bleak(  # pylint: disable=protected-access
-                    device
+                    device, advertisement_data
                 )
                 if not scan_entry.matches(prefixes, match_all=False):
                     continue
@@ -316,7 +316,7 @@ class Adapter:  # pylint: disable=too-many-instance-attributes
         # hcidump outputs the full advertisement data, assuming it's run privileged.
         # Since hcitool is privileged, we assume hcidump is too.
         # pylint: disable=consider-using-with
-        hcidump = subprocess.Popen(
+        self._hcidump = subprocess.Popen(
             ["hcidump", "--raw", "hci"],
             stdin=subprocess.DEVNULL,
             stdout=subprocess.PIPE,
@@ -332,15 +332,15 @@ class Adapter:  # pylint: disable=too-many-instance-attributes
             )
         # pylint: enable=consider-using-with
         # Throw away the first two output lines of hcidump because they are version info.
-        hcidump.stdout.readline()  # type: ignore[union-attr]
-        hcidump.stdout.readline()  # type: ignore[union-attr]
-        returncode = self._hcitool.poll()
+        self._hcidump.stdout.readline()  # type: ignore[union-attr]
+        self._hcidump.stdout.readline()  # type: ignore[union-attr]
+        returncode = self._hcidump.poll()
         start_time = time.monotonic()
         buffered: List[bytes] = []
         while returncode is None and (
             timeout is None or time.monotonic() - start_time < timeout
         ):
-            line = hcidump.stdout.readline()  # type: ignore[union-attr]
+            line = self._hcidump.stdout.readline()  # type: ignore[union-attr]
             # print(line, line[0])
             if line[0] != 32:  # 32 is ascii for space
                 if buffered:
@@ -351,11 +351,13 @@ class Adapter:  # pylint: disable=too-many-instance-attributes
                         yield parsed
                     buffered.clear()
             buffered.append(line)
-            returncode = self._hcitool.poll()
+            returncode = self._hcidump.poll()
         self.stop_scan()
 
-    async def _scan_for_interval(self, interval: float) -> Iterable[ScanEntry]:
-        """Scan advertisements for the given interval and return ScanEntry objects
+    async def _scan_for_interval(
+        self, interval: float
+    ) -> Tuple[BLEDevice, AdvertisementData]:
+        """Scan advertisements for the given interval and tuples (device, advertisement_data)
         for all advertisements heard.
         """
         if not self._scanner:
@@ -364,15 +366,21 @@ class Adapter:  # pylint: disable=too-many-instance-attributes
         await self._scanner.start()
         await asyncio.sleep(interval)
         await self._scanner.stop()
-        return await self._scanner.get_discovered_devices()
+        return self._scanner.discovered_devices_and_advertisement_data.values()
 
     def stop_scan(self) -> None:
         """Stop scanning before timeout may have occurred."""
-        if self._use_hcitool and self._hcitool:
-            if self._hcitool.returncode is None:
-                self._hcitool.send_signal(signal.SIGINT)
-                self._hcitool.wait()
-            self._hcitool = None
+        if self._use_hcitool:
+            if self._hcitool:
+                if self._hcitool.poll() is None:
+                    self._hcitool.send_signal(signal.SIGINT)
+                    self._hcitool.wait()
+                self._hcitool = None
+            if self._hcidump:
+                if self._hcidump.poll() is None:
+                    self._hcidump.send_signal(signal.SIGINT)
+                    self._hcidump.wait()
+                self._hcidump = None
         self._scanning_in_progress = False
         self._scanner = None
 
@@ -739,7 +747,7 @@ class Connection:
         """
 
         # Fetch the services.
-        bleak_services = await self.__bleak_client.get_services()
+        bleak_services = self.__bleak_client.services
 
         # pylint: disable=protected-access
         if service_uuids_whitelist:
